from .cruncher_classes import Cruncher


class SingleOptions:
    """SingleOptions"""

    def __new__(cls):
        if not hasattr(cls, "instance"):
            cls.instance = super(SingleOptions, cls).__new__(cls)
        return cls.instance

    def set_items(cls, options):
        cls.instance.options = options


class SeasonalOptions:
    def __init__(
        self,
        demetra_folder,
        java_folder,
        local_folder,
        test=False,
        verbose=False,
        replace_original_files=False,
        auto_approve=False,
        result_file_names=(
            "sa",
            "s",
            "cal",
        ),
        workspace_mode=True,
        file_name_explanation=True,
    ):
        self.demetra_folder = demetra_folder
        self.java_folder = java_folder
        self.local_folder = local_folder
        self.test = test
        self.verbose = verbose
        self.replace_original_files = replace_original_files
        self.auto_approve = auto_approve
        self.result_file_names = result_file_names
        self.workspace_mode = workspace_mode
        self.file_name_explanation = file_name_explanation
        self.set_options(workspace_mode)
        so = SingleOptions()
        so.set_items(self)

    def __repr__(self):
        template = f"""
    options= SeasonalOptions(
            demetra_folder="{self.demetra_folder}",
            java_folder="{self.java_folder}",
            local_folder="{self.local_folder}",
            test={self.test},
            verbose= {self.verbose},
            replace_original_files={self.replace_original_files},
            auto_approve={self.auto_approve},
            result_file_names={self.result_file_names},
            workspace_mode={self.workspace_mode},
    )
        """
        return template

    def set_options(self, workspace_mode):
        c = Cruncher()
        c.set_items(
            local_work_space=self.local_folder,
            crunch_folder=self.java_folder,
            demetra_folder=self.demetra_folder,
            workspace_mode=workspace_mode,
            file_name_explanation=self.file_name_explanation,
        )



from dataclasses import dataclass

from .cruncher_classes import get_cruncher

from abc import ABC, abstractmethod
from ._options import demetra_command_file_name


# get_cruncher()


@dataclass
class DemetraCaller(ABC):
    @property
    def cf(self):
        return get_cruncher().crunch_folder

    @abstractmethod
    def cruncher_command(self):
        ...

    @abstractmethod
    def demetra_command_file_name(self):
        ...

    @abstractmethod
    def exec_file_name(self, file_name):
        ...


class DemetraCallerWindows(DemetraCaller):
    def cruncher_command(self):
        return rf"start {self.cf }/jwsacruncher.bat"

    def demetra_command_file_name(self):
        return rf"{self.cf}/{demetra_command_file_name}.bat"

    def exec_file_name(self, file_name):
        return rf"{ self.cf }/{file_name}.bat"


@dataclass
class DemetraCallerLinux(DemetraCaller):
    def cruncher_command(self):
        return rf"{self.cf}/jwsacruncher"

    def demetra_command_file_name(self):
        return rf"{ self.cf}/{demetra_command_file_name}.sh"

    def exec_file_name(self, file_name):
        return rf"{self.cf}/{file_name}.sh"


@dataclass
class DemetraCallerMac(DemetraCallerLinux):
    ...



# from eseas import Options, Seasonal
# from pathlib import Path


# def test_crunch(capsys):
#     with capsys.disabled():

#         fold = "unix"
#         demetra_folder = rf"./eseas/data_for_testing/{fold}"
#         # java_folder = r"../jwsacruncher-2.2.4/bin"
#         java_folder = Path(r"../../Downloads/jwsacruncher-2.2.4/bin")
#         local_folder = r"./eseas_output"



import pandas as pd
from evdspy.EVDSlocal.common.file_classes import mock_file_items
from eseas.core.seasonal_general import SeasonalADV
from eseas.core.seasonal_options import SeasonalOptions
from eseas.core.df_operations import make_df_float
from eseas.core.cruncher_classes import get_cruncher
from eseas.core.cruncher_classes import Cruncher
from eseas.core.demetra import get_demetra_files
from eseas.core.picker_classes import OutFilePicker
from eseas.core.seas_testing_utils import get_testing_utils
from eseas.core.seas_utils import filter_xls

testing_utils = get_testing_utils()
demetra_folder = testing_utils.demetra_folder
java_folder = testing_utils.java_folder
local_folder = testing_utils.local_folder


def test_mevsimsel_general_basic():
    options = SeasonalOptions(
        demetra_folder,
        java_folder,
        local_folder,
    )
    m = SeasonalADV(options)
    m.part1()
    m.part2()


def test_a1():
    c = Cruncher()
    c.set_items(java_folder, local_folder, demetra_folder)
    dem_files = get_demetra_files(demetra_folder)
    for f in dem_files:
        of_picker = OutFilePicker(f)
        of_picker.pick_files()


def test_mevsimsel_general():
    options = SeasonalOptions(
        demetra_folder,
        java_folder,
        local_folder,
        test=False,
        verbose=False,
        replace_original_files=False,
        auto_approve=False,
        result_file_names=(
            "sa",
            "s",
            "cal",
        ),
        workspace_mode=True,
    )
    m = SeasonalADV(options)
    m.part1()
    m.part2()


def test_get_cruncher():
    c = Cruncher()
    c.set_items("", "", "")
    get_cruncher()


def test_make_df_float(capsys):
    with capsys.disabled():
        df = pd.DataFrame({"a": ["1,12", "12"], "b": [15, 20]})
        df2 = make_df_float(df)
        print(df2)


def test_filter_xls(capsys):
    with capsys.disabled():
        items = mock_file_items()
        a = filter_xls(items)
        print(a)


def test_Cruncher():
    c1 = Cruncher()
    c1.crunch_folder = "abc"
    c2 = Cruncher()
    c2.crunch_folder = "abcdefg"
    assert c1.crunch_folder == c2.crunch_folder


def test_Cruncher():
    c1 = Cruncher()
    c1.crunch_folder = "abc"
    c2 = Cruncher()
    c2.crunch_folder = "abcdefg"
    assert c1.crunch_folder == c2.crunch_folder


def test_a1():
    c = Cruncher()
    c.set_items(java_folder, local_folder, demetra_folder)
    dem_files = get_demetra_files(demetra_folder)
    for f in dem_files:
        of_picker = OutFilePicker(f)
        of_picker.pick_files()



from typing import Iterable
import functools
import traceback
import typing as t
from functools import update_wrapper
from pathlib import Path
from typing import Callable

from evdspy.EVDSlocal.utils.utils_general import replace_recursive

from .json_ops import remove_json_bad_chars


def walk(items: t.Iterable[t.Any], fnc: t.Callable) -> tuple:
    if callable(items):
        items, fnc = fnc, items

    return tuple(map(fnc, items))


def walk2(items: t.Iterable[t.Any], fnc: t.Callable) -> None:
    if callable(items):
        items, fnc = fnc, items
    _ = tuple(map(fnc, items))


def bound(f, *args):
    return update_wrapper(functools.partial(f, *args), f)


def dict_apply(dict_: dict, func: t.Callable, args=None):
    if isinstance(dict_, (tuple, list)):
        return list(map(func, dict_))
    new_dict_ = {}
    for k, v in dict_.items():
        if isinstance(v, str):
            obj = {k: func(v)}
            new_dict_.update(obj)
    return new_dict_


def list_apply(list_: Iterable, func: Callable, args=None):
    return list(func(x) for x in list_)


def tuple_apply(tuple_: tuple, func: Callable, args=None):
    return tuple(func(x) for x in tuple_)


def dict_temizle(dict_: dict, func: t.Callable = remove_json_bad_chars):
    return dict_apply(dict_, func)


def create_dir(folder: str):
    folder = Path(folder)
    if folder.is_dir():
        return
    import os

    return os.makedirs(folder)


def clean_file_name(file_name: str):
    file_name = file_name.replace("\n", " ")
    file_name = file_name.replace(":", " ")
    file_name = file_name.replace("'", " ")
    file_name = file_name.replace("$", " ")
    return file_name.replace("\\", "..__")


def deep_temizle_list(content: list) -> tuple:
    return tuple(deep_temizle(x) for x in content)


def deep_temizle(content: str) -> t.Union[str, tuple, list, dict]:
    if isinstance(
        content,
        (
            list,
            tuple,
        ),
    ):
        return deep_temizle_list(content)
    if isinstance(content, (dict)):
        return dict_apply(content, deep_temizle)
    import re

    # print(content)
    # reg1 = r'/[a-zA-Z]+/g'
    # reg2 = r'\W+'
    # reg3 = r'^[a-zA-Z0-9_ ]*$'
    reg4 = r"\W+"
    try:
        content = re.sub(reg4, "", content)
    except Exception as exc:
        print(content, type(content))
        print(exc)
    return content


def create_directory(address: Path):
    import os

    try:
        if not Path(address).is_dir():
            os.makedirs(address)
        return True
    except Exception:
        traceback.print_exc()
        return False


def make_eng(text: str) -> str:
    # return text
    dict_ = {
        "ÅŸ": "s",
        "Ã§": "c",
        "Ä±": "i",
        "Ã¼": "u",
        "Ã¶": "o",
        "ÄŸ": "g",
        "Ä°": "I",
        "Å": "S",
        "Ä": "G",
        "Ã‡": "C",
        "Ãœ": "U",
        "Ã–": "O",
    }
    return text.translate(text.maketrans(dict_))


def temizle(content: str) -> str:
    chars = ("$", "*", ">", "\xa0", "Ã ", "Ã¢", "Ä", "ÄŸ", "â€™", "'", ":")
    for harf in chars:
        content = replace_recursive(content, harf, "")
    return content


def get_os():
    import platform

    return str(platform.system()).lower()



import traceback
from pathlib import Path
from rich import print
from .seas_utils import get_absolute
from evdspy.EVDSlocal.common.file_classes import make_eng
from evdspy.EVDSlocal.utils.utils_general import replace_recursive
import time
import tempfile
import os
from typing import Union


class Cruncher:
    """cruncher"""

    crunch_folder = "."
    local_work_space = "@wspace"
    demetra_folder: str = "."

    def __new__(cls):
        if not hasattr(cls, "instance"):
            cls.instance = super(Cruncher, cls).__new__(cls)
        return cls.instance

    def set_items(
        cls,
        crunch_folder,
        local_work_space,
        demetra_folder,
        workspace_mode=False,
        file_name_explanation=True,
    ):
        """Set Items Cruncher"""
        cls.instance.crunch_folder = get_absolute(crunch_folder)
        cls.instance.local_work_space = get_absolute(local_work_space)
        cls.demetra_folder = get_absolute(demetra_folder)
        control(cls)
        cls.workspace_mode = workspace_mode
        cls.file_name_explanation = file_name_explanation
        cls.check_workspace_mode()

    def set_itemsObj(cls, obj):
        """Set Items Cruncher"""

        cls.instance.crunch_folder = get_absolute(obj.crunch_folder)
        cls.instance.local_work_space = get_absolute(obj.local_work_space)
        cls.demetra_folder = get_absolute(obj.demetra_folder)
        control(cls)
        cls.workspace_mode = obj.workspace_mode
        cls.file_name_explanation = obj.file_name_explanation
        cls.check_workspace_mode()

    def create_workspace(cls):
        def create_directory(address: Path):
            import os

            try:
                if not Path(address).is_dir():
                    os.makedirs(address)
                return True
            except Exception:
                traceback.print_exc()
                return False

        def path_str(Path_: Path):
            r = replace_recursive(str(Path_), "\\", ".")
            r = replace_recursive(r, ":", "..")
            return r

        def naming_format():
            p = Path() / cls.instance.demetra_folder
            p = path_str(p)
            p = make_eng(p)
            return f"@eseas_wspace_{p}"

        if cls.workspace_mode:
            ws = cls.instance.local_work_space
            n_fname = Path() / ws / naming_format()
            if create_directory(n_fname):
                cls.instance.local_work_space = n_fname

    def check_workspace_mode(cls):
        # Cruncher
        # TODO SingleOptions().workspace_mode
        try:
            if cls.workspace_mode:
                cls.create_workspace()
        except Exception as exc:
            print(exc)
            exit()


def control(cls):
    global checked
    if not check_cruncher(cls):

        def check(folder: str):
            if Path(folder).is_dir():
                return "exists"
            return "does not exist!"

        msg = f"""\n\n
_______________________________________________________________
Could not find some folders.
_______________________________________________________________
    java_folder     : [{check(cls.instance.crunch_folder)}]
                      {cls.instance.crunch_folder}
    local_workspace : [{check(cls.instance.local_work_space)}]
                      {cls.instance.local_work_space}
    demetra_folder  : [{check(cls.instance.demetra_folder)}]
                      {cls.instance.demetra_folder}
"""
        raise ChruncerNotSet(msg)
    else:
        checked = True


def check_write_permission(folder: Union[str, Path]) -> bool:
    if not os.path.isdir(folder):
        raise NotADirectoryError(folder)
        return False

    try:
        testfile = tempfile.TemporaryFile(dir=folder)
        testfile.close()
        return True
    except (OSError, IOError):
        return False


def check_read_permission(folder: Union[str, Path]) -> bool:
    if not os.path.isdir(folder):
        raise NotADirectoryError(folder)
        # return False
    return os.access(folder, os.R_OK)


def check_cruncher(cls):
    msg_if_error = """
    Cruncher will need write permission to
    write permission
        - local workspace
        - java_folder
    read permission
        - demetra folder
    """

    a1 = check_write_permission(Path(cls.instance.crunch_folder))
    a2 = check_write_permission(Path(cls.instance.local_work_space))
    a3 = check_read_permission(Path(cls.instance.demetra_folder))
    ok = all(
        (
            a1,
            a2,
            a3,
        )
    )
    if not ok:
        print(msg_if_error)
        time.sleep(5)
        raise ChruncerNotSet()
    return ok


class ChruncerNotSet(BaseException):
    pass


def get_cruncher():
    c = Cruncher()
    control(c)
    assert (
        c.crunch_folder is not None and c.local_work_space is not None
    ), "Chruncer not set!"
    return c


__all__ = [
    "get_cruncher",
    "Cruncher",
]



"""
----------------------------------
      Global options
----------------------------------
"""

"""
    middle_folder
    .............
      -middle_folder : 'test_output'
      local_w_space / middle_folder / ...
      # produced csv content will be placed here
"""
middle_folder = "test_output"
"""
    demetra_command_file_name
    .............
       e.g. :  demetra_commands
"""
demetra_command_file_name = "demetra_commands"
"""
    max_num
    ...............
      this number will be maximum number of xml files that can be picked
      while searching for demetra xml files.
"""
max_num = 5_000



from pathlib import Path
from evdspy.EVDSlocal.common.files import Write


def _create_general_params(folder=".", file_name="general.params"):
    content = get_general_params()
    file_name_full = Path(folder) / file_name
    if file_name_full.is_file():
        return
    return Write(file_name_full, content)


def get_general_params():
    return """<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<wsaConfig bundle="10000" csvlayout="list" csvseparator=";" ndecs="6">
    <policy>parameters</policy>
    <refreshall>true</refreshall>
    <matrix>
        <item>period</item>
        <item>span.start</item>
        <item>span.end</item>
        <item>span.n</item>
        <item>span.missing</item>
        <item>espan.start</item>
        <item>espan.end</item>
        <item>espan.n</item>
        <item>log</item>
        <item>adjust</item>
        <item>regression.lp</item>
        <item>regression.ntd</item>
        <item>regression.nmh</item>
        <item>regression.td-derived</item>
        <item>regression.td-ftest</item>
        <item>regression.easter</item>
        <item>regression.nout</item>
        <item>regression.noutao</item>
        <item>regression.noutls</item>
        <item>regression.nouttc</item>
        <item>regression.noutso</item>
        <item>regression.td(*)</item>
        <item>regression.out(*)</item>
        <item>regression.user(*)</item>
        <item>likelihood.neffectiveobs</item>
        <item>likelihood.np</item>
        <item>likelihood.logvalue</item>
        <item>likelihood.adjustedlogvalue</item>
        <item>likelihood.ssqerr</item>
        <item>likelihood.aic</item>
        <item>likelihood.aicc</item>
        <item>likelihood.bic</item>
        <item>likelihood.bicc</item>
        <item>residuals.ser</item>
        <item>residuals.ser-ml</item>
        <item>residuals.mean</item>
        <item>residuals.skewness</item>
        <item>residuals.kurtosis</item>
        <item>residuals.dh</item>
        <item>residuals.lb</item>
        <item>residuals.lb2</item>
        <item>residuals.seaslb</item>
        <item>residuals.bp</item>
        <item>residuals.bp2</item>
        <item>residuals.seasbp</item>
        <item>residuals.nudruns</item>
        <item>residuals.ludruns</item>
        <item>residuals.nruns</item>
        <item>residuals.lruns</item>
        <item>arima</item>
        <item>arima.mean</item>
        <item>arima.p</item>
        <item>arima.d</item>
        <item>arima.q</item>
        <item>arima.bp</item>
        <item>arima.bd</item>
        <item>arima.bq</item>
        <item>arima.phi(*)</item>
        <item>arima.bphi(*)</item>
        <item>arima.th(*)</item>
        <item>arima.bth(*)</item>
        <item>decomposition.seasonality</item>
        <item>decomposition.trendfilter</item>
        <item>decomposition.seasfilter</item>
        <item>m-statistics.m1</item>
        <item>m-statistics.m2</item>
        <item>m-statistics.m3</item>
        <item>m-statistics.m4</item>
        <item>m-statistics.m5</item>
        <item>m-statistics.m6</item>
        <item>m-statistics.m7</item>
        <item>m-statistics.m8</item>
        <item>m-statistics.m9</item>
        <item>m-statistics.m10</item>
        <item>m-statistics.m11</item>
        <item>m-statistics.q</item>
        <item>m-statistics.q-m2</item>
        <item>method</item>
        <item>variancedecomposition.cycle</item>
        <item>variancedecomposition.seasonality</item>
        <item>variancedecomposition.irregular</item>
        <item>variancedecomposition.tdh</item>
        <item>variancedecomposition.others</item>
        <item>variancedecomposition.total</item>
        <item>diagnostics.logstat</item>
        <item>diagnostics.levelstat</item>
        <item>diagnostics.fcast-insample-mean</item>
        <item>diagnostics.fcast-outsample-mean</item>
        <item>diagnostics.fcast-outsample-variance</item>
        <item>diagnostics.seas-lin-f</item>
        <item>diagnostics.seas-lin-qs</item>
        <item>diagnostics.seas-lin-kw</item>
        <item>diagnostics.seas-lin-friedman</item>
        <item>diagnostics.seas-lin-periodogram</item>
        <item>diagnostics.seas-lin-spectralpeaks</item>
        <item>diagnostics.seas-si-combined</item>
        <item>diagnostics.seas-si-evolutive</item>
        <item>diagnostics.seas-si-stable</item>
        <item>diagnostics.seas-res-f</item>
        <item>diagnostics.seas-res-qs</item>
        <item>diagnostics.seas-res-kw</item>
        <item>diagnostics.seas-res-friedman</item>
        <item>diagnostics.seas-res-periodogram</item>
        <item>diagnostics.seas-res-spectralpeaks</item>
        <item>diagnostics.seas-res-combined</item>
        <item>diagnostics.seas-res-combined3</item>
        <item>diagnostics.seas-res-evolutive</item>
        <item>diagnostics.seas-res-stable</item>
        <item>diagnostics.seas-i-f</item>
        <item>diagnostics.seas-i-qs</item>
        <item>diagnostics.seas-i-kw</item>
        <item>diagnostics.seas-i-periodogram</item>
        <item>diagnostics.seas-i-spectralpeaks</item>
        <item>diagnostics.seas-i-combined</item>
        <item>diagnostics.seas-i-combined3</item>
        <item>diagnostics.seas-i-evolutive</item>
        <item>diagnostics.seas-i-stable</item>
        <item>diagnostics.seas-sa-f</item>
        <item>diagnostics.seas-sa-qs</item>
        <item>diagnostics.seas-sa-kw</item>
        <item>diagnostics.seas-sa-friedman</item>
        <item>diagnostics.seas-sa-periodogram</item>
        <item>diagnostics.seas-sa-spectralpeaks</item>
        <item>diagnostics.seas-sa-combined</item>
        <item>diagnostics.seas-sa-combined3</item>
        <item>diagnostics.seas-sa-evolutive</item>
        <item>diagnostics.seas-sa-stable</item>
        <item>diagnostics.seas-sa-ac1</item>
        <item>diagnostics.td-sa-all</item>
        <item>diagnostics.td-sa-last</item>
        <item>diagnostics.td-i-all</item>
        <item>diagnostics.td-i-last</item>
        <item>diagnostics.td-res-all</item>
        <item>diagnostics.td-res-last</item>
        <item>diagnostics.ic-ratio-henderson</item>
        <item>diagnostics.ic-ratio</item>
        <item>diagnostics.msr-global</item>
        <item>diagnostics.msr(*)</item>
        <item>decomposition.parameters_cutoff</item>
        <item>decomposition.model_changed</item>
        <item>decomposition.tvar-estimator</item>
        <item>decomposition.tvar-estimate</item>
        <item>decomposition.tvar-pvalue</item>
        <item>decomposition.savar-estimator</item>
        <item>decomposition.savar-estimate</item>
        <item>decomposition.savar-pvalue</item>
        <item>decomposition.svar-estimator</item>
        <item>decomposition.svar-estimate</item>
        <item>decomposition.svar-pvalue</item>
        <item>decomposition.ivar-estimator</item>
        <item>decomposition.ivar-estimate</item>
        <item>decomposition.ivar-pvalue</item>
        <item>decomposition.tscorr-estimator</item>
        <item>decomposition.tscorr-estimate</item>
        <item>decomposition.tscorr-pvalue</item>
        <item>decomposition.ticorr-estimator</item>
        <item>decomposition.ticorr-estimate</item>
        <item>decomposition.ticorr-pvalue</item>
        <item>decomposition.sicorr-estimator</item>
        <item>decomposition.sicorr-estimate</item>
        <item>decomposition.sicorr-pvalue</item>
        <item>decomposition.ar_root(*)</item>
        <item>decomposition.ma_root(*)</item>
        <item>diagnostics.basic checks.definition:2</item>
        <item>diagnostics.basic checks.annual totals:2</item>
        <item>diagnostics.visual spectral analysis.spectral seas peaks</item>
        <item>diagnostics.visual spectral analysis.spectral td peaks</item>
        <item>diagnostics.regarima residuals.normality:2</item>
        <item>diagnostics.regarima residuals.independence:2</item>
        <item>diagnostics.regarima residuals.spectral td peaks:2</item>
        <item>diagnostics.regarima residuals.spectral seas peaks:2</item>
        <item>diagnostics.outliers.number of outliers:2</item>
        <item>diagnostics.out-of-sample.mean:2</item>
        <item>diagnostics.out-of-sample.mse:2</item>
        <item>diagnostics.seats.seas variance:2</item>
        <item>diagnostics.seats.irregular variance:2</item>
        <item>diagnostics.seats.seas/irr cross-correlation:2</item>
        <item>diagnostics.m-statistics.q:2</item>
        <item>diagnostics.m-statistics.q-m2:2</item>
        <item>diagnostics.residual trading days tests.f-test on sa (td):2</item>
        <item>diagnostics.residual trading days tests.f-test on i (td):2</item>
        <item>diagnostics.residual seasonality tests.qs test on sa:2</item>
        <item>diagnostics.residual seasonality tests.qs test on i:2</item>
        <item>diagnostics.residual seasonality tests.f-test on sa (seasonal dummies):2</item>
        <item>diagnostics.residual seasonality tests.f-test on i (seasonal dummies):2</item>
        <item>diagnostics.combined seasonality test.combined seasonality test on sa:2</item>
        <item>diagnostics.combined seasonality test.combined seasonality test on sa (last 3 years):2</item>
        <item>diagnostics.combined seasonality test.combined seasonality test on irregular:2</item>
        <item>diagnostics.quality</item>
    </matrix>
    <tsmatrix>
        <series>y</series>
        <series>y_f</series>
        <series>y_ef</series>
        <series>yc</series>
        <series>yc_f</series>
        <series>yc_ef</series>
        <series>l</series>
        <series>y_lin</series>
        <series>y_lin_f</series>
        <series>ycal</series>
        <series>ycal_f</series>
        <series>det</series>
        <series>det_f</series>
        <series>l_f</series>
        <series>l_b</series>
        <series>cal</series>
        <series>cal_f</series>
        <series>tde</series>
        <series>tde_f</series>
        <series>mhe</series>
        <series>mhe_f</series>
        <series>ee</series>
        <series>ee_f</series>
        <series>omhe</series>
        <series>omhe_f</series>
        <series>out</series>
        <series>out_f</series>
        <series>out_i</series>
        <series>out_i_f</series>
        <series>out_t</series>
        <series>out_t_f</series>
        <series>out_s</series>
        <series>out_s_f</series>
        <series>reg</series>
        <series>reg_f</series>
        <series>reg_t</series>
        <series>reg_t_f</series>
        <series>reg_s</series>
        <series>reg_s_f</series>
        <series>reg_i</series>
        <series>reg_i_f</series>
        <series>reg_sa</series>
        <series>reg_sa_f</series>
        <series>reg_y</series>
        <series>reg_y_f</series>
        <series>reg_u</series>
        <series>reg_u_f</series>
        <series>fullresiduals</series>
        <series>fcasts(?)</series>
        <series>bcasts(?)</series>
        <series>lin_fcasts(?)</series>
        <series>lin_bcasts(?)</series>
        <series>efcasts(?)</series>
        <series>decomposition.y_cmp</series>
        <series>decomposition.y_cmp_f</series>
        <series>decomposition.t_cmp</series>
        <series>decomposition.t_cmp_f</series>
        <series>decomposition.sa_cmp</series>
        <series>decomposition.s_cmp</series>
        <series>decomposition.s_cmp_f</series>
        <series>decomposition.i_cmp</series>
        <series>decomposition.a-tables.a1</series>
        <series>decomposition.a-tables.a1a</series>
        <series>decomposition.a-tables.a1b</series>
        <series>decomposition.a-tables.a6</series>
        <series>decomposition.a-tables.a7</series>
        <series>decomposition.a-tables.a8</series>
        <series>decomposition.a-tables.a8t</series>
        <series>decomposition.a-tables.a8s</series>
        <series>decomposition.a-tables.a8i</series>
        <series>decomposition.a-tables.a9</series>
        <series>decomposition.a-tables.a9sa</series>
        <series>decomposition.a-tables.a9u</series>
        <series>decomposition.a-tables.a9ser</series>
        <series>decomposition.b-tables.b1</series>
        <series>decomposition.b-tables.b2</series>
        <series>decomposition.b-tables.b3</series>
        <series>decomposition.b-tables.b4</series>
        <series>decomposition.b-tables.b5</series>
        <series>decomposition.b-tables.b6</series>
        <series>decomposition.b-tables.b7</series>
        <series>decomposition.b-tables.b8</series>
        <series>decomposition.b-tables.b9</series>
        <series>decomposition.b-tables.b10</series>
        <series>decomposition.b-tables.b11</series>
        <series>decomposition.b-tables.b12</series>
        <series>decomposition.b-tables.b13</series>
        <series>decomposition.b-tables.b14</series>
        <series>decomposition.b-tables.b15</series>
        <series>decomposition.b-tables.b16</series>
        <series>decomposition.b-tables.b17</series>
        <series>decomposition.b-tables.b18</series>
        <series>decomposition.b-tables.b19</series>
        <series>decomposition.b-tables.b20</series>
        <series>decomposition.c-tables.c1</series>
        <series>decomposition.c-tables.c2</series>
        <series>decomposition.c-tables.c3</series>
        <series>decomposition.c-tables.c4</series>
        <series>decomposition.c-tables.c5</series>
        <series>decomposition.c-tables.c6</series>
        <series>decomposition.c-tables.c7</series>
        <series>decomposition.c-tables.c8</series>
        <series>decomposition.c-tables.c9</series>
        <series>decomposition.c-tables.c10</series>
        <series>decomposition.c-tables.c11</series>
        <series>decomposition.c-tables.c12</series>
        <series>decomposition.c-tables.c13</series>
        <series>decomposition.c-tables.c14</series>
        <series>decomposition.c-tables.c15</series>
        <series>decomposition.c-tables.c16</series>
        <series>decomposition.c-tables.c17</series>
        <series>decomposition.c-tables.c18</series>
        <series>decomposition.c-tables.c19</series>
        <series>decomposition.c-tables.c20</series>
        <series>decomposition.d-tables.d1</series>
        <series>decomposition.d-tables.d2</series>
        <series>decomposition.d-tables.d3</series>
        <series>decomposition.d-tables.d4</series>
        <series>decomposition.d-tables.d5</series>
        <series>decomposition.d-tables.d6</series>
        <series>decomposition.d-tables.d7</series>
        <series>decomposition.d-tables.d8</series>
        <series>decomposition.d-tables.d9</series>
        <series>decomposition.d-tables.d10</series>
        <series>decomposition.d-tables.d10a</series>
        <series>decomposition.d-tables.d10b</series>
        <series>decomposition.d-tables.d11</series>
        <series>decomposition.d-tables.d11a</series>
        <series>decomposition.d-tables.d12</series>
        <series>decomposition.d-tables.d12a</series>
        <series>decomposition.d-tables.d13</series>
        <series>decomposition.d-tables.d14</series>
        <series>decomposition.d-tables.d15</series>
        <series>decomposition.d-tables.d16</series>
        <series>decomposition.d-tables.d16a</series>
        <series>decomposition.d-tables.d16b</series>
        <series>decomposition.d-tables.d18</series>
        <series>decomposition.d-tables.d19</series>
        <series>decomposition.d-tables.d20</series>
        <series>decomposition.e-tables.e1</series>
        <series>decomposition.e-tables.e2</series>
        <series>decomposition.e-tables.e3</series>
        <series>decomposition.e-tables.e11</series>
        <series>t</series>
        <series>t_f</series>
        <series>sa</series>
        <series>sa_f</series>
        <series>s</series>
        <series>s_f</series>
        <series>i</series>
        <series>i_f</series>
        <series>benchmarking.original</series>
        <series>benchmarking.target</series>
        <series>benchmarking.result</series>
        <series>decomposition.y_lin</series>
        <series>decomposition.y_lin_f</series>
        <series>decomposition.y_lin_ef</series>
        <series>decomposition.t_lin</series>
        <series>decomposition.t_lin_f</series>
        <series>decomposition.t_lin_e</series>
        <series>decomposition.t_lin_ef</series>
        <series>decomposition.sa_lin</series>
        <series>decomposition.sa_lin_f</series>
        <series>decomposition.sa_lin_e</series>
        <series>decomposition.sa_lin_ef</series>
        <series>decomposition.s_lin</series>
        <series>decomposition.s_lin_f</series>
        <series>decomposition.s_lin_e</series>
        <series>decomposition.s_lin_ef</series>
        <series>decomposition.i_lin</series>
        <series>decomposition.i_lin_f</series>
        <series>decomposition.i_lin_e</series>
        <series>decomposition.i_lin_ef</series>
        <series>decomposition.sa_cmp_f</series>
        <series>decomposition.i_cmp_f</series>
        <series>decomposition.i_cmp_e</series>
        <series>decomposition.t_cmp_e</series>
        <series>decomposition.s_cmp_e</series>
        <series>decomposition.sa_cmp_e</series>
        <series>decomposition.i_cmp_ef</series>
        <series>decomposition.t_cmp_ef</series>
        <series>decomposition.s_cmp_ef</series>
        <series>decomposition.sa_cmp_ef</series>
        <series>decomposition.si_cmp</series>
    </tsmatrix>
</wsaConfig>"""



from eseas.core.seasonal_general import SeasonalADV as Seasonal
from eseas.core.seasonal_general import SeasonalADV
from eseas.core.seasonal_options import SeasonalOptions as Options
from eseas.core.seasonal_options import SeasonalOptions
from eseas.core.cruncher_classes import Cruncher
from eseas.data_for_testing.some_data import get_sample_data
from eseas.core.utils_general2 import walk, walk2


__all__ = [
    "Cruncher",
    "SeasonalADV",
    "SeasonalOptions",
    "Seasonal",
    "Options",
    "get_sample_data",
    "walk",
    "walk2",
]



import pandas as pd
from pathlib import Path


def get_rand_hash(num=5) -> str:
    import secrets

    return secrets.token_urlsafe(nbytes=num)


def try_to_write_excel(df: pd.DataFrame, file_name: Path) -> None:
    try:
        df.to_excel(file_name)
    except Exception:
        pa = file_name.parent
        hash_ = get_rand_hash(5)
        file_name_new = file_name.stem + f"_{hash_}.xlsx"
        print(Path(pa) / file_name_new)
        df.to_excel(Path(pa) / file_name_new)


def rreplace(st: str, e, y) -> str:
    if e not in st:
        return st
    st = st.replace(e, y)
    return rreplace(st, e, y)


def sayi_donustur(pot_sayi: str) -> float:
    if not pot_sayi:
        return pot_sayi
    try:
        if isinstance(pot_sayi, float):
            return pot_sayi
        if str(pot_sayi).isnumeric():
            return float(pot_sayi)
        pot_sayi = str(pot_sayi)
        pot_sayi = rreplace(pot_sayi, ".", "")
        pot_sayi = pot_sayi.replace(",", ".")
        if not pot_sayi.replace(".", "").isnumeric():
            return pot_sayi
        return float(pot_sayi)
    except Exception:
        return pot_sayi


def convert_df_number(df: pd.DataFrame, except_columns=()):
    def convert_number_item(item):
        try:
            new_number = sayi_donustur(item)  # float(str(item).replace(",", "."))
        except Exception as exc:
            print(item)
            print(exc)
            new_number = 0
            # new_number = -987654321.0123
            # raise exc
        return new_number

    def convert_numbers(numbers):
        return tuple(map(convert_number_item, numbers))

    for column in list(df.columns):
        if column not in except_columns:
            df[column] = convert_numbers(list(df[column]))
        else:
            ...
            # print("passing...", column)
    return df


def make_df_float(df):
    except_columns = ("donem",)
    df = convert_df_number(df, except_columns)
    return df


def test_make_float():
    data_dict = {
        "donem": ["200201", "200202"],
        "kolon1": ["60,8456", "66,12656"],
        "kolon2": ["60,8456", "66,12656"],
    }
    df = pd.DataFrame.from_records(data_dict)
    new_df = make_df_float(df)
    print(new_df.head())



import functools
import random
import re
from .utils_general2 import list_apply, tuple_apply, dict_apply

strategies = {"strict": r"\W+", "light": r"^[a-zA-Z0-9_ ]", "medium": r"\W+ "}
valid_chars_JSON = (
    """ . """ "d",
    "a",
    "t",
    "f",
    "i",
    "h",
    "r",
    "s",
    "=",
    "'",
    "{",
    '"',
    "b",
    "e",
    "_",
    "p",
    "y",
    "o",
    ":",
    " ",
    "[",
    "S",
    "E",
    "R",
    "I",
    "C",
    "O",
    "D",
    "T",
    "P",
    "A",
    "1",
    "2",
    ",",
    "G",
    "U",
    "N",
    "M",
    "Y",
    "L",
    "K",
    "ÄŸ",
    "Ä±",
    "l",
    "k",
    "m",
    "F",
    "z",
    "H",
    "W",
    "g",
    "v",
    "n",
    "Q",
    "B",
    "V",
    "0",
    "5",
    "9",
    "8",
    "}",
    "3",
    "7",
    "Ãœ",
    "4",
    "ÅŸ",
    "u",
    "c",
    "]",
    "Ä°",
    "Å",
    "Z",
    "Ã¼",
    "q",
    "6",
    "w",
    "Ã§",
    "Ã¶",
    "x",
    "Ã‡",
    "X",
    "Ä",
    "J",
    "j",
    "Ã–",
    "Ã¢",
    "Ã ",
    " ",
    "-",
)


def base_replace(string, level="strict"):
    bound_func = functools.update_wrapper(
        functools.partial(base_replace, level=level), base_replace
    )
    if isinstance(string, (list,)):
        return list_apply(string, bound_func)
    if isinstance(string, (tuple,)):
        return tuple_apply(string, bound_func)
    if isinstance(string, (dict,)):
        return dict_apply(string, bound_func)
    reg_strategy = strategies[level]
    return re.sub(reg_strategy, "_", string)


def replace_strict(string):
    return base_replace(string, "strict")


def replace_light(string):
    return base_replace(string, "light")


def replace_medium(string):
    return base_replace(string, "medium")



import sys
from pathlib import Path

from evdspy.EVDSlocal.common.file_classes import FileItem

from .seas_utils import view_display


def this_is_pytest() -> bool:
    """
    Sometimes we need to check if it is pytest
    in order to avoid tests to break
    """

    def check():
        path_ = Path(sys.argv[0])
        return "pytest" in str(path_.stem)

    if len(sys.argv) > 0 and check():
        return True
    return False


def get_input_both(msg="Your choice", default_answer="y") -> str:
    """
    using input function causes pytest to raise error
    we need to check if it is pytest
    """
    view_display(msg)
    if this_is_pytest():
        return default_answer
    return input()


def get_input_from_user() -> bool:
    msg = """
==========================================================================
    Confirm running `jwsacruncher` for seasonal adjusment ?
           ( if you prefer running manually instead, you may do so by calling
             batch file that was created by this script )

==========================================================================
?(y/N)
    """
    ans = get_input_both(msg, "y")
    if str(ans).lower() in (
        "y",
        "yes",
    ):
        return True
    else:
        return False


def display(items: list[FileItem]) -> tuple:
    def print_spec(item: FileItem):
        template = f"""
---------------- Detail -------------------------
dir       : {item.dir_path}
file_name : {item.file_name}
-------------------------------------------------
        """
        view_display(template)
        return template

    return tuple(map(print_spec, items))


def common_space_msg(test, demetra_folder):
    return f"""
========================================================================
        common_space_check has run
test : {test}
folder to check demetra files : {demetra_folder}
========================================================================
"""


def seasonal_results_msg(test):
    return f"""
========================================================================
        seasonal_results function is running.
        test : {test}
        Collecting results
========================================================================
    """


def get_results_info(adres):
    t = f"""
    --------------------------
    {adres}
    """
    return t
